{% extends "base_layout.html" %}
{% from "_formhelpers.html" import render_field %}
{% block body %}
      <div class="main-content container-fluid">
          <div class="row">
            <div class="col-xs-offset-2 col-md-offset-4 col-lg-offset-4 col-xs-8 col-md-4 col-lg-4">
              <div class="panel panel-default panel-border-color panel-border-color-primary">
                <div class="panel-heading panel-heading-divider">Packy Agent Activation</div>
                <div class="panel-body">
                    <form method="post" novalidate="" class="form-horizontal group-border-dashed">
                      {{ form.csrf_token }}
                      {{ form.email()|safe }}
                      {{ form.password()|safe }}
                      {{ form.extra(value='yes')|safe }}
                      {{ render_field(form.agent) }}
                      <div>
                        <p class="text-center">
                          <button type="submit" class="btn btn-space btn-primary">Continue</button>
                        </p>
                      </div>
                    </form>
                </div>
              </div>
            </div>
          </div>
      </div>
{% endblock %}
