server {
    listen 0.0.0.0:{{ console_http_port }};
    charset utf-8;

    location / {
        uwsgi_pass 127.0.0.1:8002;
        include uwsgi_params;
        uwsgi_param HTTP_X_FORWARDED_FOR $proxy_add_x_forwarded_for;
        uwsgi_param HTTP_X_REAL_IP $remote_addr;
        uwsgi_param HTTP_X_FORWARDED_PROTO $http_x_forwarded_proto;
    }
}
