{% extends "base_layout.html" %}
{% block body %}
<div>Something went wrong, please, <a href="javascript:window.location.href=window.location.href">try again</a> later.</div>
{% if settings.is_debug_mode() %}
  </div>
    <span></span><a href="{{ url_for('debug') }}">See debug information</a> (shown only in debug mode)</span>
  <div>
{% endif %}
{% endblock %}