{% extends "layout.html" %}
{% block content %}
          <div class="row">
            <div class="col-xs-12 col-md-6 col-lg-6">
              <div class="panel panel-default panel-border-color panel-border-color-primary">
                <div class="panel-heading panel-heading-divider">Agent Control</div>
                <div class="panel-body">
                        <div class="widget widget-tile">
                          <div class="data-info">
                            <div class="desc">Worker status</div>
                            <div class="value">{{ worker_status }}</div>
                          </div>
                        </div>

                        <form method="post" novalidate="" class="form-horizontal group-border-dashed">
                          {{ form.csrf_token }}
                        <p class="text-center">
                          <button type="submit" name="action" value="start_worker" class="btn btn-space btn-primary">Start Worker</button>
                          <button type="submit" name="action" value="stop_worker" class="btn btn-space btn-primary">Stop Worker</button>
                          <button type="submit" name="action" value="restart_worker" class="btn btn-space btn-primary">Restart Worker</button>
                          <button type="submit" name="action" value="restart_console" class="btn btn-space btn-primary">Restart Console</button>
                          <button type="submit" name="action" value="restart" class="btn btn-space btn-primary">Restart All</button>
                          <button type="submit" name="action" value="reboot" class="btn btn-space btn-primary"
                                  onclick="return confirm('Are you sure you want to reboot?');">Reboot</button>
                        </p>
                        </form>
                </div>
              </div>
            </div>
          </div>
{% endblock %}